# Gepetto Viewer to Blender export

Script for importing in blender a motion viewed with gepetto viewer

* [Exporting the scene](#exporting-the-scene)
* [Recording frames](#recording-frames)
* [Loading the motion into blender](#loading-the-motion-into-blender)

## Exporting the scene

### URDF to blender conversion - suggested approach

Exporting from OpenSceneGraph does not always work. One can also convert URDF files to a script loadable by blender:
`blender/urdf_to_blender.py -p prefix/ -i robot.urdf -o robot_blend.py`
where `prefix` is the prefix you gave to Gepetto Viewer. _A trailing slash must be added_.

### Robot - moving objects
Robots have to be exported **before** applying any configuration. Right after having loaded all your models, do `gui.writeNodeFile (nodeName, filename)`

### Environment - static objects
Environment has to be exported **after** in place (having been placed correctly). Once they are at their good location, do
`gui.writeNodeFile (nodeName, filename)`
If you export them before placing them, you can capture them in a frame using the next method, in which case you will have
1 differents `path.yaml` files (one with N frames for the robot and one with 1 frame for the environment).

### OSG export results by file format
- Collada: seems OK, but quality seems poor.
- OBJ: not tested
- Box, Cylinders and other primitives: not supported by dae and obj file format

## Recording frames

First, setup the export by calling `gui.setCaptureTransform (filename, nodeName)` where `nodeName` refers to a node containing all the objects that will be recorded.

- Frame by frame record:
a frame can then be saved using `gui.captureTransform ()` to capture the current state.
- Capturing on refresh:
use `gui.captureTransformOnRefresh(true)` to start capturing and `gui.captureTransformOnRefresh(false)` to stop it.

**Notice**: datas will be appended to `filename` so it must be empty at the beginning. Otherwise, you will get an error when loading the file into blender.

```python
gui.setCaptureTransform ("path.yaml", "hrp2")

# Frame by Frame
for i in range(0,100):
	   #update robot configuration
     ...
        gui.refresh ()
        gui.captureTransform ()


# Capture on refresh
gui.captureTransformOnRefresh(true)
for i in range(0,100):
	   #update robot configuration
     ...
        gui.refresh ()

gui.captureTransformOnRefresh(false)

```

## Loading the motion into blender

### Running blender

Blender is using python 3, so you may encounter some problems of you do not set `PYTHONPATH` environment properly.

Three options:

- ```blender --python <source-folder>/gepetto-viewer-corba/blender/gepettoimport.py```
- Install it as an addon (https://www.blender.org/manual/advanced/scripting/python/add_ons.html) - save the user preferences
- Manually load the plugin by running the script `blender/gepettoimport.py`
_Switch to the script view in blender (menu on the top the screen) the python script that you can find in the gepetto-viewer under `blender/gepettoimport.py` and press `Run script`_

Personnally, I'm using the following script to launch blender:
```bash
#!/bin/bash

export PYTHONPATH=/usr/lib/python3/dist-packages
blender-2.75a-linux-glibc211-i686/blender $@ --python "/local/jmirabel/devel/hpp/src/gepetto-viewer-corba/blender/gepettoimport.py"
```

### Load the motion

- Load your object definition files of the robot and the environment.
  * import collada or obj files from `File` menu
  * import the script generated by `urdf_to_blender.py` by pressing `Space Bar` and searching for "Import a script generated with urdf_to_blend.py"
- Press `Space bar` and search for "Import a YAML Gepetto Viewer path file"
- Select the file containing the configurations and wait...

Voilà! A set of frames should automatically be generated, resulting in an animation.

If some errors occur, the messages will appear in the console, or the terminal used to launch blender, depending on how the script was launched.
